package main

import (
	"fmt"
	tm "github.com/buger/goterm"
	"sort"
	"time"
)

type ActiveCalls []TrunksEvent

func (calls *ActiveCalls) isKnownCall(event TrunksEvent) (bool, int) {
	for idx, dlg := range *calls {
		if dlg.ID == event.ID {
			return true, idx
		}
	}

	return false, -1
}

// Remove terminated calls after 3 seconds
func (calls *ActiveCalls) purgeCalls() {
	purgeIndex := -1

	for idx, dlg := range *calls {
		if dlg.Event != "Terminated" {
			continue
		}
		if dlg.Time+3 > time.Now().Unix() {
			continue
		}

		purgeIndex = idx
		break
	}

	if purgeIndex != -1 {
		*calls = append((*calls)[:purgeIndex], (*calls)[purgeIndex+1:]...)
		calls.purgeCalls()
	}

	return
}

func (calls *ActiveCalls) newCall(event TrunksEvent) {
	if event.Event == "Terminated" {
		// Do not add Terminated calls
		return
	}

	*calls = append(*calls, event)
	return
}

func (calls *ActiveCalls) updateCall(event TrunksEvent, index int) {
	// Update status only if higher
	updatedCallStatus, _ := event.getCallStatus()
	currentCallStatus, _ := (*calls)[index].getCallStatus()
	if updatedCallStatus > currentCallStatus {
		(*calls)[index].Event = event.Event
		(*calls)[index].Time = event.Time
	}

	// Remaining fields: update only empty fields
	// (necessary for initial load when a update is processed before initial status)
	if (*calls)[index].CallID == "" {
		(*calls)[index].CallID = event.CallID
	}
	if (*calls)[index].Brand == "" {
		(*calls)[index].Brand = event.Brand
	}
	if (*calls)[index].Company == "" {
		(*calls)[index].Company = event.Company
	}
	if (*calls)[index].Direction == "" {
		(*calls)[index].Direction = event.Direction
	}
	if (*calls)[index].Carrier == "" {
		(*calls)[index].Carrier = event.Carrier
	}
	if (*calls)[index].Ddiprovider == "" {
		(*calls)[index].Ddiprovider = event.Ddiprovider
	}
	if (*calls)[index].Caller == "" {
		(*calls)[index].Caller = event.Caller
	}
	if (*calls)[index].Callee == "" {
		(*calls)[index].Callee = event.Callee
	}
}

func (calls ActiveCalls) print(lines int, platforms int) {
	tm.Clear() // Clear current screen
	tm.MoveCursor(1, 1)

	totals := tm.NewTable(0, 10, 5, ' ', 0)
	if platforms > 1 {
		fmt.Fprintf(totals, "##\tDuration\tDirection\tStatus\tCaller\tCallee\tPlatform\tBrand\tClient\tCarrier\n")
		fmt.Fprintf(totals, "--\t---------\t--------\t------\t------\t------\t--------\t-----\t------\t-------\n")
	} else {
		fmt.Fprintf(totals, "##\tDuration\tDirection\tStatus\tCaller\tCallee\tBrand\tClient\tCarrier\n")
		fmt.Fprintf(totals, "--\t---------\t--------\t------\t------\t------\t-----\t------\t-------\n")
	}

	sort.Slice(calls, func(i, j int) bool {
		return calls[i].Time < calls[j].Time
	})

	if len(calls) <= lines {
		// Print all rows
		for i, dlg := range calls {
			r := dlg.ToTableRow()
			if platforms > 1 {
				fmt.Fprintf(totals, "%2d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", i+1, r.Duration, r.Direction, r.CallState, r.Caller, r.Callee, r.Platform, r.Brand, r.Client, r.Carrier)
			} else {
				fmt.Fprintf(totals, "%2d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", i+1, r.Duration, r.Direction, r.CallState, r.Caller, r.Callee, r.Brand, r.Client, r.Carrier)
			}
			i++
		}
	} else {
		// Some rows will be hidden
		for i := 1; i <= lines/2; i++ {
			r := calls[i].ToTableRow()
			if platforms > 1 {
				fmt.Fprintf(totals, "%2d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", i, r.Duration, r.Direction, r.CallState, r.Caller, r.Callee, r.Platform, r.Brand, r.Client, r.Carrier)
			} else {
				fmt.Fprintf(totals, "%2d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", i, r.Duration, r.Direction, r.CallState, r.Caller, r.Callee, r.Brand, r.Client, r.Carrier)
			}
		}

		if platforms > 1 {
			fmt.Fprintf(totals, "--\t---------\t--------\t----------\t------\t------\t-----\t------\t-------\n")
		} else {
			fmt.Fprintf(totals, "--\t---------\t--------\t------\t------\t------\t-----\t------\t-------\n")
		}
		for i := len(calls) - lines/2; i < len(calls); i++ {
			r := calls[i].ToTableRow()
			if platforms > 1 {
				fmt.Fprintf(totals, "%2d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", i+1, r.Duration, r.Direction, r.CallState, r.Caller, r.Callee, r.Platform, r.Brand, r.Client, r.Carrier)
			} else {
				fmt.Fprintf(totals, "%2d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", i+1, r.Duration, r.Direction, r.CallState, r.Caller, r.Callee, r.Brand, r.Client, r.Carrier)
			}
		}
	}

	tm.Println(totals)
	tm.Print("Active calls: ", len(calls))
	tm.Flush()
}
