package main

import (
	"flag"
	"fmt"
	"strings"
	"time"
)

type Filter struct {
	Direction string
	Brand     string
	Client    string
	Carrier   string
}

type SourcesFlag map[string]string

func (s SourcesFlag) String() string {
	srcs := make([]string, 0)

	for name, addr := range s {
		srcs = append(srcs, name+":"+addr)
	}

	return strings.Join(srcs, ",")
}

func (s SourcesFlag) Set(st string) error {
	platforms := strings.Split(st, ",")
	monoPlatform := len(platforms) == 1

	for _, platform := range platforms {
		var platformName, platformAddr string

		platformElems := strings.Split(platform, ":")

		if len(platformElems) == 1 && monoPlatform {
			platformElems = append(platformElems, platformElems[0])
			platformElems[0] = "local"
		} else if len(platformElems) != 2 {
			return fmt.Errorf("error parsing %s flag", st)
		}

		platformName = platformElems[0]
		platformAddr = platformElems[1] + ":26379"

		s[platformName] = platformAddr
	}
	return nil
}

func main() {
	activeCalls := make(ActiveCalls, 0)
	c := make(chan TrunksEvent)
	sources := make(SourcesFlag)

	// Parse flags
	flag.Var(sources, "s", "Platform name and IP/hostname of IvozProvider data HA host (empty for local platform), multiple values supported (e.g. AplatformName:AplatformSQLHA,BplatformName:BplatformSQLHA)")
	direction := flag.String("d", "", "Filter by Direction (inbound/outbound)")
	brand := flag.String("b", "", "Show only calls from brand matching given pattern")
	client := flag.String("c", "", "Show only calls from client matching given pattern")
	carrier := flag.String("cr", "", "Show only calls from carrier matching given pattern")
	lines := flag.Int("l", 40, "Maximum num of rows")
	flag.Parse()

	if len(sources) == 0 {
		sources.Set("data.ivozprovider.local")
	}

	if *direction != "inbound" && *direction != "outbound" {
		*direction = ""
	}

	filter := Filter{
		Direction: *direction,
		Brand:     *brand,
		Client:    *client,
		Carrier:   *carrier,
	}

	for platformName, platformAddr := range sources {
		// Obtain redis master address
		redisMasterAddr := getRedisMasterAddr(platformAddr, "mymaster")

		// Obtain initial picture and new events
		go initialPicture(platformName, redisMasterAddr, c)
		go subscribeForUpdates(platformName, redisMasterAddr, c)
	}

	ticker := time.NewTicker(time.Second)

	for {
		select {
		case event := <-c:
			if isKnown, idx := activeCalls.isKnownCall(event); isKnown {
				activeCalls.updateCall(event, idx)
			} else {
				if event.matchesFilter(filter) {
					activeCalls.newCall(event)
				}
			}
		case <-ticker.C:
			activeCalls.purgeCalls()
			activeCalls.print(*lines, len(sources))
		}
	}
}
