package main

import (
	"strings"
	"time"
)

type TrunksEvent struct {
	Platform    string
	Event       string `json:"Event"`
	Time        int64  `json:"Time"`
	CallID      string `json:"Call-ID"`
	Brand       string `json:"Brand"`
	Company     string `json:"Company"`
	Direction   string `json:"Direction"`
	Carrier     string `json:"Carrier"`
	Ddiprovider string `json:"DdiProvider"`
	Caller      string `json:"Caller"`
	Callee      string `json:"Callee"`
	ID          string `json:"ID"`
}

type TableRow struct {
	Platform  string
	Direction string
	Duration  string
	CallState string
	Caller    string
	Callee    string
	Brand     string
	Client    string
	Carrier   string
}

func (te TrunksEvent) ToTableRow() TableRow {
	tableRow := TableRow{
		Platform:  te.Platform,
		Direction: te.Direction,
		Duration:  time.Since(time.Unix(te.Time, 0)).Truncate(time.Second).String(),
		CallState: "",
		Caller:    te.Caller,
		Callee:    te.Callee,
		Brand:     te.Brand,
		Client:    te.Company,
		Carrier:   "",
	}

	_, callState := te.getCallStatus()
	tableRow.CallState = strings.ToUpper(callState)

	if te.Carrier != "" {
		tableRow.Carrier = te.Carrier
	} else if te.Ddiprovider != "" {
		tableRow.Carrier = te.Ddiprovider
	} else {
		tableRow.Carrier = "-"
	}

	return tableRow
}

// Trying (0) -> Proceeding (1) -> Early (2) -> Confirmed (3) -> Terminated (4)
func (te TrunksEvent) getCallStatus() (int, string) {
	switch te.Event {
	case "Trying":
		return 0, "Calling"
	case "Proceeding":
		return 1, "Calling"
	case "Early":
		return 2, "Ringing"
	case "Confirmed":
		return 3, "In Call"
	default:
		// "Terminated"
		return 4, "Hangup"
	}
}

func (te *TrunksEvent) matchesFilter(f Filter) bool {
	if f.Direction != "" && te.Direction != f.Direction {
		return false
	}

	if f.Brand != "" && !strings.Contains(strings.ToLower(te.Brand), strings.ToLower(f.Brand)) {
		return false
	}

	if f.Client != "" && !strings.Contains(strings.ToLower(te.Company), strings.ToLower(f.Client)) {
		return false
	}

	if f.Carrier != "" {
		if te.Carrier != "" && !strings.Contains(strings.ToLower(te.Carrier), strings.ToLower(f.Carrier)) {
			return false
		}
		if te.Ddiprovider != "" && !strings.Contains(strings.ToLower(te.Ddiprovider), strings.ToLower(f.Carrier)) {
			return false
		}
		if te.Ddiprovider == "" && te.Carrier == "" {
			return false
		}
	}

	return true
}
